==> alpha-0.7.5.7-src.tar.bz2: Alpha-0.7.5.7-src/Alpha/ExecuteCommandDlg.cpp <==
// ExecuteCommandDlg.cpp
// (c) 2003 exeal

#include "StdAfx.h"
#include "resource.h"
#include "ExecuteCommandDlg.h"
#include "Alpha.h"
#include "..\Manah\Text.h"
using Alpha::CAlphaApp;
using Alpha::CExecuteCommandDlg;
using Manah::Windows::CDosCommand;
using namespace Manah;
using namespace Manah::Text;


// CExecuteCommandDlg class implementation
/////////////////////////////////////////////////////////////////////////////

///	RXgN^
CExecuteCommandDlg::CExecuteCommandDlg(CAlphaApp* pApplication)
		: m_pApplication(pApplication), m_bExecuting(false) {
	assert(m_pApplication != 0);
//	m_oDosCommand.SetTimeout(5000);
}

///	@see	CDialog::OnCancel
void CExecuteCommandDlg::OnCancel() {
	if(m_bExecuting) {	// [~] {^
		m_dosCommand.Stop();
		::SetWindowTextW(GetDlgItem(IDCANCEL), L"");
	} else
		CDialog::OnCancel();
}

///	@see	CDialog::OnClose
void CExecuteCommandDlg::OnClose() {
	m_dosCommand.Stop();
	m_bHijackConsole = IsDlgButtonChecked(IDC_CHK_GETCONSOLE) == BST_CHECKED;
	m_bUseUnicode = IsDlgButtonChecked(IDC_CHK_USEUNICODEFORINPUT) == BST_CHECKED;

	CDialog::OnClose();
}

///	@see	CDialog::OnCommand
bool CExecuteCommandDlg::OnCommand(WORD wID, WORD wNotifyCode, HWND hwndCtrl) {
	switch(wID) {
	case IDC_COMBO_COMMAND:
		if(wNotifyCode == CBN_EDITCHANGE)
			::EnableWindow(GetDlgItem(IDOK), ::GetWindowTextLength(GetDlgItem(wID)) != 0);
		break;
	case IDC_BTN_CLEAR:	// [NA]
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_SETSEL, 0, -1);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, false, reinterpret_cast<LPARAM>(L""));
		break;
	}

	return CDialog::OnCommand(wID, wNotifyCode, hwndCtrl);
}

///	@see	CDialog::OnInitDialog
bool CExecuteCommandDlg::OnInitDialog(HWND hwndFocus, LPARAM lInitParam) {
	CDialog::OnInitDialog(hwndFocus, lInitParam);

	CheckDlg2StateButton(IDC_CHK_GETCONSOLE, m_bHijackConsole);
	CheckDlg2StateButton(IDC_CHK_USEUNICODEFORINPUT, m_bUseUnicode);
	OnCommand(IDC_COMBO_COMMAND, CBN_EDITCHANGE, 0);

	return true;
}

///	@see	CDialog::OnOK
void CExecuteCommandDlg::OnOK() {
	if(!m_bExecuting) {	// [s]
		HWND			hwndCmdLineCmb;
		wchar_t*		pwszCmdLine = 0;
		size_t			cchCmdLine;
		wstring			strCmdLine;			// `̃R}hC
		const wstring&	strFilePath = m_pApplication->GetDocumentManager().GetActiveDocument()->GetFilePath();

		hwndCmdLineCmb = GetDlgItem(IDC_COMBO_COMMAND);
		cchCmdLine = ::GetWindowTextLength(hwndCmdLineCmb);
		pwszCmdLine = new wchar_t[cchCmdLine + 1];
		::GetWindowText(hwndCmdLineCmb, pwszCmdLine, cchCmdLine + 1);

		// "$F" t@CpXɕϊ
		strCmdLine = pwszCmdLine;
		wstring::size_type	iLast = 0;
		while(true) {
			iLast = strCmdLine.find(L"$F", iLast);
			if(iLast == wstring::npos)
				break;
			strCmdLine.replace(iLast, 2, L"\"" + strFilePath + L"\"");
			iLast += strFilePath.length() + 2;
		}

		// [R\[荞] ꍇ̓EBhEpӂ
		if(IsDlgButtonChecked(IDC_CHK_GETCONSOLE) == BST_CHECKED) {
			HWND	hSplitStatic = GetDlgItem(IDC_STATIC_PROMPT);
			HWND	hInputEdit = GetDlgItem(IDC_EDIT_INPUT);
			HWND	hOutputEdit = GetDlgItem(IDC_EDIT_OUTPUT);
			RECT	rect;

			::EnableWindow(GetDlgItem(IDC_COMBO_COMMAND), false);
			::EnableWindow(hInputEdit, true);
			SetDlgItemText(IDOK, L"(&I)");
			SetDlgItemText(IDCANCEL, L"~(&A)");
			GetWindowRect(rect);
			rect.bottom = rect.top + 460;
			MoveWindow(rect, true);
			::EnableWindow(GetDlgItem(IDC_CHK_GETCONSOLE), false);
			::EnableWindow(GetDlgItem(IDC_CHK_USEUNICODEFORINPUT), false);
			::MoveWindow(hSplitStatic, 10, 122, 466, 2, false);
			::MoveWindow(hInputEdit, 10, 130, 466, 18, false);
			::MoveWindow(hOutputEdit, 10, 150, 466, 270, false);
			::ShowWindow(hSplitStatic, SW_SHOW);
			::ShowWindow(hInputEdit, SW_SHOW);
			::ShowWindow(hOutputEdit, SW_SHOW);

			// R}h̎s
			m_bExecuting = true;
			m_dosCommand.Execute(strCmdLine, true, static_cast<CDosCommand::ICommandEvent*>(this));
			::SetFocus(hInputEdit);
			::SendMessage(hInputEdit, EM_SETSEL, 0, -1);
		} else
			m_dosCommand.Execute(strCmdLine, false);

		delete[] pwszCmdLine;
	} else {	// []
		HWND		hwndInputEdit = GetDlgItem(IDC_EDIT_INPUT);
		char*		pszInput = 0;
		wchar_t*	pwszInput = 0;
		size_t		cchInput = ::GetWindowTextLength(hwndInputEdit);
		size_t		cchConverted;
		string		strInput;

		pwszInput = new wchar_t[cchInput + 1];
		::GetWindowText(hwndInputEdit, pwszInput, cchInput + 1);
		if(IsDlgButtonChecked(IDC_CHK_USEUNICODEFORINPUT) == BST_CHECKED)	// [W͂ Unicode gp]
			strInput.assign(reinterpret_cast<char*>(pwszInput), cchInput * 2);
		else {
			pszInput = new char[cchInput * 2 + 1];
			cchConverted = ConvertUnicodeToShiftJIS(pszInput, pwszInput);
			strInput.assign(pszInput, cchConverted);
			delete[] pszInput;
		}
		m_dosCommand.SendLine(strInput);
		delete[] pwszInput;
		::SetFocus(hwndInputEdit);
		::SendMessage(hwndInputEdit, EM_SETSEL, 0, -1);
	}

//	CDialog::OnOK();
}

///	@see IDosCommandEvent::OnExit
void CExecuteCommandDlg::OnExit(CDosCommand::ExecuteStatus es) {
	if(!IsWindow())	// sɃoc{^Ƃɗ
		return;

	SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_SETSEL, -1, -1);
	switch(es) {
	case CDosCommand::timeout:
		MessageBox(L"^CAEg܂B\nR}h͒~܂B", IDS_APPNAME, MB_ICONEXCLAMATION);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, 0,
			reinterpret_cast<LPARAM>(L"\r\n(R}h~܂B)\r\n"));
		break;
	case CDosCommand::aborted:
		MessageBox(L"R}h~܂B", IDS_APPNAME, MB_ICONEXCLAMATION);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, 0,
			reinterpret_cast<LPARAM>(L"\r\n(R}h~܂B)\r\n"));
		break;
	case CDosCommand::failedToCreateNewProcess:
		MessageBox(L"vZX̋NɎs܂!", IDS_APPNAME, MB_ICONHAND);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, 0,
			reinterpret_cast<LPARAM>(L"\r\n(R}h̎sɎs܂B)\r\n"));
		break;
	case CDosCommand::failedToCreateNewThread:
		MessageBox(L"Xbh̋NɎs܂!", IDS_APPNAME, MB_ICONHAND);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, 0,
			reinterpret_cast<LPARAM>(L"\r\n(R}h̎sɎs܂B)\r\n"));
		break;
	}
	m_bExecuting = false;

	::EnableWindow(GetDlgItem(IDC_CHK_GETCONSOLE), true);
	::EnableWindow(GetDlgItem(IDC_CHK_USEUNICODEFORINPUT), true);
	::EnableWindow(GetDlgItem(IDC_COMBO_COMMAND), true);
	::EnableWindow(GetDlgItem(IDC_EDIT_INPUT), false);
	::SetFocus(GetDlgItem(IDC_COMBO_COMMAND));
	SendDlgItemMessage(IDC_COMBO_COMMAND, CB_SETEDITSEL, 0, -1);
	SetDlgItemText(IDOK, L"s(&E)");
	SetDlgItemText(IDCANCEL, L"");
	SetWindowText(L"R}h̎s");
}

///	@see IDosCommandEvent::OnReceiveOutput
void CExecuteCommandDlg::OnReceiveOutput(const string& strOutput) {
	stringstream	ssOutput;
	int				cchOutput;
	wchar_t*		pwszOutput = 0;

	// LF ȂGfBbgRg[̂߂ CR+LF ɕϊ
	// (̓̓Rg[gꍇ͕ϊȂĂ낤)
	string::size_type	i = 0, iLast = 0;
	while(true) {
		i = strOutput.find('\n', iLast);
		if(i == string::npos) {
			ssOutput << strOutput.substr(iLast);
			break;
		} else if(i == 0)
			ssOutput << "\r\n";
		else if(strOutput[i - 1] == '\r')
			ssOutput << strOutput.substr(iLast, i - iLast + 1);
		else
			ssOutput << strOutput.substr(iLast, i - iLast) << "\r\n";
		iLast = i + 1;
	}

	// ݂ ANSI R[hy[WƂO Unicode ɕϊ
	cchOutput = ::MultiByteToWideChar(CP_ACP, 0, ssOutput.str().c_str(), -1, 0, 0);
	pwszOutput = new wchar_t[cchOutput * sizeof(wchar_t)];
	::MultiByteToWideChar(CP_ACP, 0, ssOutput.str().c_str(), -1, pwszOutput, cchOutput);

	// ɒǉ
	SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_SETSEL, -1, -1);
	SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, false, reinterpret_cast<LPARAM>(pwszOutput));
	delete[] pwszOutput;
}

/* [EOF] */

==> alpha-0.7.5.16-src.tar.bz2: Alpha-0.7.5.16-src/Alpha/ExecuteCommandDlg.cpp <==
// ExecuteCommandDlg.cpp
// (c) 2003-2005 exeal

#include "StdAfx.h"
#include "ExecuteCommandDlg.h"
#include "Alpha.h"
using Alpha::CAlphaApp;
using Alpha::CExecuteCommandDlg;
using Manah::Windows::CDosCommand;
using namespace Manah;
using namespace std;


// ̃NX͏폜


///	RXgN^
CExecuteCommandDlg::CExecuteCommandDlg(CAlphaApp& app) : m_app(app), m_bExecuting(false) {
//	m_dosCommand.SetTimeout(5000);
}

///	@see	CDialog::OnCancel
void CExecuteCommandDlg::OnCancel() {
	if(m_bExecuting) {	// [~] {^
		m_dosCommand.Stop();
		::SetWindowTextW(GetDlgItem(IDCANCEL), L"");
	} else
		CDialog::OnCancel();
}

///	@see	CDialog::OnClose
void CExecuteCommandDlg::OnClose() {
	m_dosCommand.Stop();
	m_bHijackConsole = IsDlgButtonChecked(IDC_CHK_GETCONSOLE) == BST_CHECKED;
	m_bUseUnicode = IsDlgButtonChecked(IDC_CHK_USEUNICODEFORINPUT) == BST_CHECKED;

	CDialog::OnClose();
}

///	@see	CDialog::OnCommand
bool CExecuteCommandDlg::OnCommand(WORD wID, WORD wNotifyCode, HWND hwndCtrl) {
	switch(wID) {
	case IDC_COMBO_COMMAND:
		if(wNotifyCode == CBN_EDITCHANGE)
			::EnableWindow(GetDlgItem(IDOK), ::GetWindowTextLength(GetDlgItem(wID)) != 0);
		break;
	case IDC_BTN_CLEAR:	// [NA]
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_SETSEL, 0, -1);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, false, reinterpret_cast<LPARAM>(L""));
		break;
	}

	return CDialog::OnCommand(wID, wNotifyCode, hwndCtrl);
}

///	@see	CDialog::OnInitDialog
bool CExecuteCommandDlg::OnInitDialog(HWND hwndFocus, LPARAM lInitParam) {
	CDialog::OnInitDialog(hwndFocus, lInitParam);

	CheckDlg2StateButton(IDC_CHK_GETCONSOLE, m_bHijackConsole);
	CheckDlg2StateButton(IDC_CHK_USEUNICODEFORINPUT, m_bUseUnicode);
	OnCommand(IDC_COMBO_COMMAND, CBN_EDITCHANGE, 0);

	return true;
}

///	@see	CDialog::OnOK
void CExecuteCommandDlg::OnOK() {
	if(!m_bExecuting) {	// [s]
		HWND			hwndCmdLineCmb = GetDlgItem(IDC_COMBO_COMMAND);
		const int		cchCmdLine = ::GetWindowTextLength(hwndCmdLineCmb);
		wchar_t*		pwszCmdLine = new wchar_t[cchCmdLine + 1];
		const wchar_t*	pwszFilePath = m_app.GetActiveBuffer().GetPathName();
		const wstring	strFilePath = (pwszFilePath != 0) ? wstring(L"\"") + pwszFilePath + L"\"" : L"";

		::GetWindowText(hwndCmdLineCmb, pwszCmdLine, cchCmdLine + 1);

		// "$F" t@CpXɕϊ
		wstring	strCmdLine = pwszCmdLine;
		wstring::size_type	iLast = 0;
		while(true) {
			iLast = strCmdLine.find(L"$F", iLast);
			if(iLast == wstring::npos)
				break;
			strCmdLine.replace(iLast, 2, strFilePath);
			iLast += strFilePath.length() + 2;
		}
		delete[] pwszCmdLine;

		// [R\[荞] ꍇ̓EBhEpӂ
		if(IsDlgButtonChecked(IDC_CHK_GETCONSOLE) == BST_CHECKED) {
			HWND	hSplitStatic = GetDlgItem(IDC_STATIC_PROMPT);
			HWND	hInputEdit = GetDlgItem(IDC_EDIT_INPUT);
			HWND	hOutputEdit = GetDlgItem(IDC_EDIT_OUTPUT);
			RECT	rect;

			::EnableWindow(GetDlgItem(IDC_COMBO_COMMAND), false);
			::EnableWindow(hInputEdit, true);
			SetDlgItemText(IDOK, L"(&I)");
			SetDlgItemText(IDCANCEL, L"~(&A)");
			GetWindowRect(rect);
			rect.bottom = rect.top + 460;
			MoveWindow(rect, true);
			::EnableWindow(GetDlgItem(IDC_CHK_GETCONSOLE), false);
			::EnableWindow(GetDlgItem(IDC_CHK_USEUNICODEFORINPUT), false);
			::MoveWindow(hSplitStatic, 10, 122, 466, 2, false);
			::MoveWindow(hInputEdit, 10, 130, 466, 18, false);
			::MoveWindow(hOutputEdit, 10, 150, 466, 270, false);
			::ShowWindow(hSplitStatic, SW_SHOW);
			::ShowWindow(hInputEdit, SW_SHOW);
			::ShowWindow(hOutputEdit, SW_SHOW);

			// R}h̎s
			m_bExecuting = true;
			m_dosCommand.Execute(strCmdLine, true, static_cast<CDosCommand::ICommandEvent*>(this));
			::SetFocus(hInputEdit);
			::SendMessage(hInputEdit, EM_SETSEL, 0, -1);
		} else
			m_dosCommand.Execute(strCmdLine, false);
	} else {	// []
		HWND		hwndInputEdit = GetDlgItem(IDC_EDIT_INPUT);
		const int	cchInput = ::GetWindowTextLength(hwndInputEdit);
		wchar_t*	pwszInput = new wchar_t[cchInput + 1];
		string		strInput;

		::GetWindowText(hwndInputEdit, pwszInput, cchInput + 1);
		if(IsDlgButtonChecked(IDC_CHK_USEUNICODEFORINPUT) == BST_CHECKED)	// [W͂ Unicode gp]
			strInput.assign(reinterpret_cast<char*>(pwszInput), cchInput * sizeof(wchar_t));
		else {
			const int	cb = ::WideCharToMultiByte(CP_ACP, WC_DEFAULTCHAR, pwszInput, cchInput, 0, 0, 0, 0);
			char* const	pszInput = new char[cb];
			::WideCharToMultiByte(CP_ACP, WC_DEFAULTCHAR, pwszInput, cchInput, pszInput, cb, 0, 0);
			strInput.assign(pszInput, cb);
			delete[] pszInput;
		}
		m_dosCommand.SendLine(strInput);
		delete[] pwszInput;
		::SetFocus(hwndInputEdit);
		::SendMessage(hwndInputEdit, EM_SETSEL, 0, -1);
	}

//	CDialog::OnOK();
}

///	@see IDosCommandEvent::OnExit
void CExecuteCommandDlg::OnExit(CDosCommand::ExecuteStatus es) {
	if(!IsWindow())	// sɃoc{^Ƃɗ
		return;

	SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_SETSEL, -1, -1);
	switch(es) {
	case CDosCommand::timeout:
		MessageBox(L"^CAEg܂B\nR}h͒~܂B", IDS_APPNAME, MB_ICONEXCLAMATION);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, 0,
			reinterpret_cast<LPARAM>(L"\r\n(R}h~܂B)\r\n"));
		break;
	case CDosCommand::aborted:
		MessageBox(L"R}h~܂B", IDS_APPNAME, MB_ICONEXCLAMATION);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, 0,
			reinterpret_cast<LPARAM>(L"\r\n(R}h~܂B)\r\n"));
		break;
	case CDosCommand::failedToCreateNewProcess:
		MessageBox(L"vZX̋NɎs܂!", IDS_APPNAME, MB_ICONHAND);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, 0,
			reinterpret_cast<LPARAM>(L"\r\n(R}h̎sɎs܂B)\r\n"));
		break;
	case CDosCommand::failedToCreateNewThread:
		MessageBox(L"Xbh̋NɎs܂!", IDS_APPNAME, MB_ICONHAND);
		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, 0,
			reinterpret_cast<LPARAM>(L"\r\n(R}h̎sɎs܂B)\r\n"));
		break;
	}
	m_bExecuting = false;

	::EnableWindow(GetDlgItem(IDC_CHK_GETCONSOLE), true);
	::EnableWindow(GetDlgItem(IDC_CHK_USEUNICODEFORINPUT), true);
	::EnableWindow(GetDlgItem(IDC_COMBO_COMMAND), true);
	::EnableWindow(GetDlgItem(IDC_EDIT_INPUT), false);
	::SetFocus(GetDlgItem(IDC_COMBO_COMMAND));
	SendDlgItemMessage(IDC_COMBO_COMMAND, CB_SETEDITSEL, 0, -1);
	SetDlgItemText(IDOK, L"s(&E)");
	SetDlgItemText(IDCANCEL, L"");
	SetWindowText(L"R}h̎s");
}

///	@see IDosCommandEvent::OnReceiveOutput
void CExecuteCommandDlg::OnReceiveOutput(const string& strOutput) {
	// LF ȂGfBbgRg[̂߂ CR+LF ɕϊ
	// (̓̓Rg[gꍇ͕ϊȂĂ낤)
	stringstream	ssOutput;
	string::size_type	i = 0, iLast = 0;
	while(true) {
		i = strOutput.find('\n', iLast);
		if(i == string::npos) {
			ssOutput << strOutput.substr(iLast);
			break;
		} else if(i == 0)
			ssOutput << "\r\n";
		else if(strOutput[i - 1] == '\r')
			ssOutput << strOutput.substr(iLast, i - iLast + 1);
		else
			ssOutput << strOutput.substr(iLast, i - iLast) << "\r\n";
		iLast = i + 1;
	}

	// ݂ ANSI R[hy[WƂO Unicode ɕϊ
	const int		cchOutput = ::MultiByteToWideChar(CP_ACP, 0, ssOutput.str().c_str(), -1, 0, 0);
	const wchar_t*	pwszOutput = new wchar_t[cchOutput * sizeof(wchar_t)];

	::MultiByteToWideChar(CP_ACP, 0, ssOutput.str().c_str(), -1, const_cast<wchar_t*>(pwszOutput), cchOutput);

	// ɒǉ
	SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_SETSEL, -1, -1);
	SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, false, reinterpret_cast<LPARAM>(pwszOutput));
	delete[] pwszOutput;
}

/* [EOF] */
